@@ -14,7 +14,7 @@
 from xmptools.xmptools import makeFileURI, isContainerItemPredicate, makeContainerItemPredicate
 from xmptools.xmptools import cbd, identity
 from xmptools.xmptools import XMPMetadata
-from xmptools.xmptools import DC, XMP, EXIF, CRS
+from xmptools.xmptools import DC, DCT, XMP, EXIF, CRS, PHOTOSHOP
 from xmptools.xmptools import JPEG_EXTENSIONS, TIFF_EXTENSIONS, RAW_EXTENSIONS, DNG_EXTENSIONS
 from xmptools.xmptools import XMP_EXTENSIONS
 from xmptools.xmptools import FileTypeError, XMPParser, XMPSerializer
@@ -23,7 +23,7 @@ __all__ = [
     'makeFileURI', 'isContainerItemPredicate', 'makeContainerItemPredicate',
     'cbd', 'identity',
     'XMPMetadata',
-    'DC', 'XMP', 'EXIF', 'CRS',
+    'DC', 'DCT', 'XMP', 'EXIF', 'CRS', 'PHOTOSHOP',
     'JPEG_EXTENSIONS', 'TIFF_EXTENSIONS', 'RAW_EXTENSIONS', 'DNG_EXTENSIONS',
     'XMP_EXTENSIONS',
     'FileTypeError', 'XMPParser', 'XMPSerializer'
