@@ -20,35 +20,40 @@ from rdflib.plugins.serializers.rdfxml import PrettyXMLSerializer
 from xml.sax import make_parser, handler, SAXParseException
 import io
 import re
-from datetime import datetime
+from datetime import date, datetime, tzinfo, timedelta
 import os.path
 import pathlib
 from urllib.parse import urlparse, unquote_plus
 
-TOOLNAME = "So Many Aircraft xmptools 0.1"
+TOOLNAME = "So Many Aircraft xmptools 0.4"
 XMP_NS = "adobe:ns:meta/"
 XMP_NAME = "xmpmeta"
 XMP_TAG_OPEN = '<x:{0}'.format(XMP_NAME).encode('utf-8')
 XMP_TAG_CLOSE = '</x:{0}>'.format(XMP_NAME).encode('utf-8')
-XMP_TAG_OPEN_FULL =\
+XMP_TAG_OPEN_FULL = \
     '<x:{0} xmlns:x="{1}" x:xmptk="{2}">\n'.format(XMP_NAME, XMP_NS, TOOLNAME).encode('utf-8')
 
+
 class FileTypeError(Error):
     def __init__(self, message, path, original=None):
         super().__init__(message.format(path, original))
         self.path = path
         self.original = original
 
+
 LI_MATCH_PATTERN = re.compile(str(RDF) + "_([0-9]+)")
 LI_CREATE_PATTERN = str(RDF) + "_{0}"
 
+
 def isContainerItemPredicate(uri):
     match = LI_MATCH_PATTERN.match(uri)
     return int(match.group(1)) if match else None
 
+
 def makeContainerItemPredicate(index):
     return LI_CREATE_PATTERN.format(index)
 
+
 class XMPHandler(RDFXMLHandler):
     def reset(self):
         super().reset()
@@ -70,6 +75,7 @@ class XMPHandler(RDFXMLHandler):
             self.error("Repeated node-elements: %s" % "".join(name))
         self.parent.object = self.current.subject
 
+
 class XMPParser(Parser):
     def __init__(self):
         self._parser = None
@@ -92,6 +98,7 @@ class XMPParser(Parser):
         except SAXParseException as e:
             raise FileTypeError('Possibly not XMP because "{1}"', None, original=e)
 
+
 class XMPSerializer(PrettyXMLSerializer):
     # We must subclass PrettyXMLSerializer, because the Adobe XMP toolkit expects
     # blank nodes to be serialized "in line" and not as separate Descriptions. The
@@ -125,6 +132,7 @@ class XMPSerializer(PrettyXMLSerializer):
         # Replace actual container item predicates with <rdf:li> as per the XMP spec
         super().predicate(self.RDFLI if isContainerItemPredicate(pred) else pred, obj, depth)
 
+
 plugin.register("xmp", Parser, "xmptools", "XMPParser")
 plugin.register("xmp", Serializer, "xmptools", "XMPSerializer")
 
@@ -135,15 +143,19 @@ DC = Namespace("http://purl.org/dc/elements/1.1/")
 DCT = Namespace("http://purl.org/dc/terms/")
 PHOTOSHOP = Namespace("http://ns.adobe.com/photoshop/1.0/")
 
+
 class MoreThanOneValue(Error):
     pass
 
+
 def URI(u):
     return u if isinstance(u, URIRef) or u is None else URIRef(u)
 
+
 def makeFileURI(path):
     return URIRef(pathlib.Path(os.path.abspath(path)).as_uri())
 
+
 def makeFilePath(uri, scheme="file"):
     u = urlparse(uri)
     if u.scheme == scheme:
@@ -151,6 +163,7 @@ def makeFilePath(uri, scheme="file"):
     else:
         raise ValueError("URI does not have '{}' scheme: {}".format(scheme, str(uri)))
 
+
 def getContainerStatements(graph, source, predicate):
     containers = list(graph.objects(URI(source), predicate))
     n = len(containers)
@@ -162,6 +175,7 @@ def getContainerStatements(graph, source, predicate):
     else:
         raise MoreThanOneValue("Expected only one value for {0}".format(predicate))
 
+
 def adjustTriple(triple, node_map, check_predicates=False):
     s, p, o = triple
     for node in node_map:
@@ -173,6 +187,7 @@ def adjustTriple(triple, node_map, check_predicates=False):
             o = node_map[node]
     return s, p, o
 
+
 def adjustNodes(node_map, source_graph, destination_graph=None, check_predicates=False):
     if destination_graph is None:
         destination_graph = type(source_graph)()
@@ -183,6 +198,7 @@ def adjustNodes(node_map, source_graph, destination_graph=None, check_predicates
         destination_graph.sourceIsXMP = False
     return destination_graph
 
+
 def adjustNodesInPlace(node_map, graph, check_predicates=False):
     for triple in list(graph.triples((None, None, None))):
         new_triple = adjustTriple(triple, node_map, check_predicates=check_predicates)
@@ -193,9 +209,11 @@ def adjustNodesInPlace(node_map, graph, check_predicates=False):
         graph.url = node_map[graph.url]
     return graph
 
+
 def identity(x):
     return x
 
+
 def cbd(source, target, resource):
     # Does not support reified statements (yet)
     subjects = list([resource])
@@ -208,13 +226,73 @@ def cbd(source, target, resource):
                 subjects.append(leaf)
     return target
 
+
+class iso8601:
+    patterns =\
+        {'iso8601': (("(?P<y>[0-9]{4})-(?P<mo>[0-9]{2})-(?P<d>[0-9]{2})"
+                      "(?:T(?P<h>[0-9]{2}):(?P<m>[0-9]{2}):(?P<s>[0-9]{2})(?:[.](?P<f>[0-9]+))?"
+                      "(?P<z>[-+Z](?:(?P<oh>[0-9]{2}):(?P<om>[0-9]{2}))?)?)?$"),
+                     True),
+         'exif':    (("(?P<y>[0-9]{4}):(?P<mo>[0-9]{2}):(?P<d>[0-9]{2})"
+                      "(?:[ ](?P<h>[0-9]{2}):(?P<m>[0-9]{2}):(?P<s>[0-9]{2}))?"),
+                     False)}
+
+    def __init__(self):
+        self.re = dict()
+        self.tz = dict()
+        for key in self.patterns:
+            (pattern, self.tz[key]) = self.patterns[key]
+            self.re[key] = re.compile(pattern)
+
+    def parse(self, string):
+        for key in self.re:
+            match = self.re[key].match(string)
+            if match:
+                (y, mo, d, h, m, s, f) = match.group("y", "mo", "d", "h", "m", "s", "f")
+                if not h:
+                    return date(int(y), int(mo), int(d))
+                else:
+                    fraction = 0
+                    if f:
+                        n = len(f)
+                        if n > 6:
+                            raise ValueError("At most microsecond precision is allowed: ." + f)
+                        else:
+                            fraction = int(f) * 10 ** (6 - n)
+                    if self.tz[key]:
+                        (z, oh, om) = match.group("z", "oh", "om")
+                        return datetime(int(y), int(mo), int(d), int(h), int(m), int(s), fraction,
+                                        timezone(0 if z[0] == 'Z' else oh,
+                                                 0 if z[0] == 'Z' else om,
+                                                 z[0] == '-'))
+                    else:
+                        return datetime(int(y), int(mo), int(d), int(h), int(m), int(s), fraction,
+                                        timezone())
+        return None
+
+
+class timezone(tzinfo):
+    def __init__(self, oh=0, om=0, negative=False):
+        m = (int(oh) if oh else 0) * 60 + (int(om) if om else 0)
+        self.offset = timedelta(minutes=(-m if negative else m))
+
+    def utcoffset(self, dt):
+        return self.offset
+
+    def dst(self, dt):
+        return None
+
+
 JPEG_EXTENSIONS = [".jpg", ".jpeg", ".JPG", ".JPEG"]
 TIFF_EXTENSIONS = [".tif", ".tiff", ".TIF", ".TIFF"]
-DNG_EXTENSIONS  = [".dng", ".DNG"]
-RAW_EXTENSIONS  = [".cr2", ".CR2", ".cr3", ".CR3"]  # obviously wholly incomplete still
-XMP_EXTENSIONS  = [".xmp", ".XMP"]
+DNG_EXTENSIONS = [".dng", ".DNG"]
+RAW_EXTENSIONS = [".cr2", ".CR2", ".cr3", ".CR3"]  # obviously wholly incomplete still
+XMP_EXTENSIONS = [".xmp", ".XMP"]
+
 
 class XMPMetadata(Graph):
+    iso8601 = iso8601()
+
     def __init__(self, url=None):
         self.initialized = False
         if url is None:
@@ -321,8 +399,9 @@ class XMPMetadata(Graph):
     def getDate(self, predicate=XMP.MetadataDate):
         value = self.getValue(predicate)
         if value:
-            date = value.value
-            return datetime.fromisoformat(date) if isinstance(date, str) else date
+            date_candidate = value.value
+            return self.iso8601.parse(date_candidate) if isinstance(date_candidate,
+                                                                    str) else date_candidate
         else:
             return None
 
